def get_version(filename):
    from re import findall
    with open(filename) as f:
        metadata = dict(findall(r"__([a-z]+)__ = '([^']+)'", f.read()))
    return metadata['version']

project = 'uritools'
copyright = '2014-2016 Thomas Kemmer'
version = get_version(b'../uritools/__init__.py')
release = version

extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.coverage',
    'sphinx.ext.doctest',
    'sphinx.ext.todo'
]
exclude_patterns = ['_build']
master_doc = 'index'
html_theme = 'default'



from __future__ import unicode_literals

import collections
import ipaddress
import re

from .encoding import uridecode

_URI_COMPONENTS = ('scheme', 'authority', 'path', 'query', 'fragment')


def _ip_literal(address):
    # RFC 3986 3.2.2: In anticipation of future, as-yet-undefined IP
    # literal address formats, an implementation may use an optional
    # version flag to indicate such a format explicitly rather than
    # rely on heuristic determination.
    #
    #  IP-literal = "[" ( IPv6address / IPvFuture  ) "]"
    #
    #  IPvFuture  = "v" 1*HEXDIG "." 1*( unreserved / sub-delims / ":" )
    #
    # If a URI containing an IP-literal that starts with "v"
    # (case-insensitive), indicating that the version flag is present,
    # is dereferenced by an application that does not know the meaning
    # of that version flag, then the application should return an
    # appropriate error for "address mechanism not supported".
    if isinstance(address, bytes):
        address = address.decode('ascii')
    if address.startswith('v'):
        raise ValueError('address mechanism not supported')
    return ipaddress.IPv6Address(address)


def _ipv4_address(address):
    try:
        if isinstance(address, bytes):
            return ipaddress.IPv4Address(address.decode('ascii'))
        else:
            return ipaddress.IPv4Address(address)
    except ValueError:
        return None


class SplitResult(collections.namedtuple('SplitResult', _URI_COMPONENTS)):
    """Base class to hold :func:`urisplit` results."""

    __slots__ = ()  # prevent creation of instance dictionary

    @property
    def userinfo(self):
        authority = self.authority
        if authority is None:
            return None
        userinfo, present, _ = authority.rpartition(self.AT)
        if present:
            return userinfo
        else:
            return None

    @property
    def host(self):
        authority = self.authority
        if authority is None:
            return None
        _, _, hostinfo = authority.rpartition(self.AT)
        host, _, port = hostinfo.rpartition(self.COLON)
        if port.lstrip(self.DIGITS):
            return hostinfo
        else:
            return host

    @property
    def port(self):
        authority = self.authority
        if authority is None:
            return None
        _, present, port = authority.rpartition(self.COLON)
        if present and not port.lstrip(self.DIGITS):
            return port
        else:
            return None

    def geturi(self):
        """Return the re-combined version of the original URI as a string."""
        scheme, authority, path, query, fragment = self

        # RFC 3986 5.3. Component Recomposition
        result = []
        if scheme is not None:
            result.extend([scheme, self.COLON])
        if authority is not None:
            result.extend([self.SLASH, self.SLASH, authority])
        result.append(path)
        if query is not None:
            result.extend([self.QUEST, query])
        if fragment is not None:
            result.extend([self.HASH, fragment])
        return self.EMPTY.join(result)

    def getscheme(self, default=None):
        """Return the URI scheme in canonical (lowercase) form, or `default`
        if the original URI did not contain a scheme component.

        """
        scheme = self.scheme
        if scheme is None:
            return default
        elif isinstance(scheme, bytes):
            return scheme.decode('ascii').lower()
        else:
            return scheme.lower()

    def getuserinfo(self, default=None, encoding='utf-8', errors='strict'):
        """Return the decoded userinfo subcomponent of the URI authority, or
        `default` if the original URI did not contain a userinfo
        field.

        """
        userinfo = self.userinfo
        if userinfo is None:
            return default
        else:
            return uridecode(userinfo, encoding, errors)

    def gethost(self, default=None):
        """Return the decoded host subcomponent of the URI authority as a
        string or an :mod:`ipaddress` address object, or `default` if
        the original URI did not contain a host.

        """
        host = self.host
        if host is None or (not host and default is not None):
            return default
        elif host.startswith(self.LBRACKET) and host.endswith(self.RBRACKET):
            return _ip_literal(host[1:-1])
        elif host.startswith(self.LBRACKET) or host.endswith(self.RBRACKET):
            raise ValueError('Invalid host %r' % host)  # FIXME: remove?
        else:
            return _ipv4_address(host) or uridecode(host, 'utf-8').lower()

    def getport(self, default=None):
        """Return the port subcomponent of the URI authority as an
        :class:`int`, or `default` if the original URI did not contain
        a port or if the port was empty.

        """
        port = self.port
        if port:
            return int(port)
        else:
            return default

    def getpath(self, encoding='utf-8', errors='strict'):
        """Return the normalized decoded URI path."""
        path = self.__remove_dot_segments(self.path)
        return uridecode(path, encoding, errors)

    def getquery(self, default=None, encoding='utf-8', errors='strict'):
        """Return the decoded query string, or `default` if the original URI
        did not contain a query component.

        """
        query = self.query
        if query is None:
            return default
        else:
            return uridecode(query, encoding, errors)

    def getquerydict(self, encoding='utf-8', errors='strict'):
        """Split the query component into individual `name=value` pairs and
        return a dictionary of query variables.  The dictionary keys
        are the unique query variable names and the values are lists
        of values for each name.

        """
        dict = collections.defaultdict(list)
        for name, value in self.getquerylist(encoding, errors):
            dict[name].append(value)
        return dict

    def getquerylist(self, encoding='utf-8', errors='strict'):
        """Split the query component into individual `name=value` pairs and
        return a list of `(name, value)` tuples.

        """
        if self.query:
            qsl = [self.query]
        else:
            return []
        for sep in self.QUERYSEP:
            qsl = [s for qs in qsl for s in qs.split(sep) if s]
        items = []
        for qs in qsl:
            parts = qs.partition(self.EQ)
            name = uridecode(parts[0], encoding, errors)
            if parts[1]:
                value = uridecode(parts[2], encoding, errors)
            else:
                value = None
            items.append((name, value))
        return items

    def getfragment(self, default=None, encoding='utf-8', errors='strict'):
        """Return the decoded fragment identifier, or `default` if the
        original URI did not contain a fragment component.

        """
        fragment = self.fragment
        if fragment is None:
            return default
        else:
            return uridecode(fragment, encoding, errors)

    def transform(self, ref, strict=False):
        """Transform a URI reference relative to `self` into a
        :class:`SplitResult` representing its target URI.

        """
        scheme, authority, path, query, fragment = self.RE.match(ref).groups()

        # RFC 3986 5.2.2. Transform References
        if scheme is not None and (strict or scheme != self.scheme):
            path = self.__remove_dot_segments(path)
        elif authority is not None:
            scheme = self.scheme
            path = self.__remove_dot_segments(path)
        elif not path:
            scheme = self.scheme
            authority = self.authority
            path = self.path
            query = self.query if query is None else query
        elif path.startswith(self.SLASH):
            scheme = self.scheme
            authority = self.authority
            path = self.__remove_dot_segments(path)
        else:
            scheme = self.scheme
            authority = self.authority
            path = self.__remove_dot_segments(self.__merge(path))
        return type(self)(scheme, authority, path, query, fragment)

    def __merge(self, path):
        # RFC 3986 5.2.3. Merge Paths
        if self.authority is not None and not self.path:
            return self.SLASH + path
        else:
            parts = self.path.rpartition(self.SLASH)
            return parts[1].join((parts[0], path))

    @classmethod
    def __remove_dot_segments(cls, path):
        # RFC 3986 5.2.4. Remove Dot Segments
        pseg = []
        for s in path.split(cls.SLASH):
            if s == cls.DOT:
                continue
            elif s != cls.DOTDOT:
                pseg.append(s)
            elif len(pseg) == 1 and not pseg[0]:
                continue
            elif pseg and pseg[-1] != cls.DOTDOT:
                pseg.pop()
            else:
                pseg.append(s)
        # adjust for trailing '/.' or '/..'
        if path.rpartition(cls.SLASH)[2] in (cls.DOT, cls.DOTDOT):
            pseg.append(cls.EMPTY)
        if path and len(pseg) == 1 and pseg[0] == cls.EMPTY:
            pseg.insert(0, cls.DOT)
        return cls.SLASH.join(pseg)


class SplitResultBytes(SplitResult):

    __slots__ = ()  # prevent creation of instance dictionary

    # RFC 3986 Appendix B
    RE = re.compile(br"""
    (?:([^:/?#]+):)?        # scheme
    (?://([^/?#]*))?        # authority
    ([^?#]*)                # path
    (?:\?([^#]*))?          # query
    (?:\#(.*))?             # fragment
    """, flags=re.VERBOSE)

    # RFC 3986 2.2 gen-delims
    COLON, SLASH, QUEST, HASH, LBRACKET, RBRACKET, AT = (
        b':', b'/', b'?', b'#', b'[', b']', b'@'
    )

    # RFC 3986 3.3 dot-segments
    DOT, DOTDOT = b'.', b'..'

    EMPTY, EQ = b'', b'='

    DIGITS = b'0123456789'

    QUERYSEP = (b';', b'&')


class SplitResultString(SplitResult):

    __slots__ = ()  # prevent creation of instance dictionary

    # RFC 3986 Appendix B
    RE = re.compile(r"""
    (?:([^:/?#]+):)?        # scheme
    (?://([^/?#]*))?        # authority
    ([^?#]*)                # path
    (?:\?([^#]*))?          # query
    (?:\#(.*))?             # fragment
    """, flags=re.VERBOSE)

    # RFC 3986 2.2 gen-delims
    COLON, SLASH, QUEST, HASH, LBRACKET, RBRACKET, AT = ':/?#[]@'

    # RFC 3986 3.3 dot-segments
    DOT, DOTDOT = '.', '..'

    EMPTY, EQ = '', '='

    DIGITS = '0123456789'

    QUERYSEP = ';&'


def urisplit(uristring):
    """Split a well-formed URI string into a tuple with five components
    corresponding to a URI's general structure::

      <scheme>://<authority>/<path>?<query>#<fragment>

    """
    if isinstance(uristring, bytes):
        result = SplitResultBytes
    else:
        result = SplitResultString
    return result(*result.RE.match(uristring).groups())


def uriunsplit(parts):
    """Combine the elements of a five-item iterable into a URI string."""
    scheme, authority, path, query, fragment = parts
    if isinstance(path, bytes):
        result = SplitResultBytes
    else:
        result = SplitResultString
    return result(scheme, authority, path, query, fragment).geturi()



from string import hexdigits

from .chars import UNRESERVED

try:
    _fromhex = bytes.fromhex
except AttributeError:
    def _fromhex(x): return chr(int(x, 16))

if isinstance(chr(0), bytes):
    _fromint = chr
else:
    def _fromint(i): return bytes([i])


# RFC 3986 2.1: For consistency, URI producers and normalizers should
# use uppercase hexadecimal digits for all percent-encodings.
def _pctenc(byte):
    return ('%%%02X' % byte).encode()

_unreserved = frozenset(memoryview(UNRESERVED.encode('ascii')).tolist())

_encoded = {
    b'': [_fromint(i) if i in _unreserved else _pctenc(i) for i in range(256)]
}

_decoded = {
    (a + b).encode(): _fromhex(a + b) for a in hexdigits for b in hexdigits
}


def uriencode(uristring, safe='', encoding='utf-8', errors='strict'):
    """Encode a URI string or string component."""
    if isinstance(uristring, bytes):
        values = memoryview(uristring).tolist()
    else:
        values = memoryview(uristring.encode(encoding, errors)).tolist()
    if not isinstance(safe, bytes):
        safe = safe.encode('ascii')
    try:
        encode = _encoded[safe].__getitem__
    except KeyError:
        enclist = _encoded[b''][:]
        for i in memoryview(safe).tolist():
            enclist[i] = _fromint(i)
        _encoded[safe] = enclist
        encode = enclist.__getitem__
    return b''.join(map(encode, values))


def uridecode(uristring, encoding='utf-8', errors='strict'):
    """Decode a URI string or string component."""
    if isinstance(uristring, bytes):
        parts = uristring.split(b'%')
    else:
        parts = uristring.encode(encoding or 'ascii', errors).split(b'%')
    result = [parts[0]]
    append = result.append
    decode = _decoded.get
    for s in parts[1:]:
        append(decode(s[:2], b'%' + s[:2]))
        append(s[2:])
    if encoding is not None:
        return b''.join(result).decode(encoding, errors)
    else:
        return b''.join(result)



from __future__ import unicode_literals

import unittest

from uritools import urisplit


class SplitTest(unittest.TestCase):

    def check(self, uri, parts, decoded=None):
        result = urisplit(uri)
        self.assertEqual(result, parts, 'Error parsing %r' % uri)
        self.assertEqual(result.geturi(), uri, 'Error recomposing %r' % uri)

    def test_rfc3986(self):
        """urisplit test cases from [RFC3986] 3. Syntax Components"""
        cases = [
            ('foo://example.com:8042/over/there?name=ferret#nose',
             ('foo', 'example.com:8042', '/over/there', 'name=ferret',
              'nose')),
            ('urn:example:animal:ferret:nose',
             ('urn', None, 'example:animal:ferret:nose', None, None)),
            (b'foo://example.com:8042/over/there?name=ferret#nose',
             (b'foo', b'example.com:8042', b'/over/there', b'name=ferret',
              b'nose')),
            (b'urn:example:animal:ferret:nose',
             (b'urn', None, b'example:animal:ferret:nose', None, None)),
        ]
        for uri, parts in cases:
            self.check(uri, parts)

    def test_abnormal(self):
        cases = [
            ('', (None, None, '', None, None)),
            (':', (None, None, ':', None, None)),
            (':/', (None, None, ':/', None, None)),
            ('://', (None, None, '://', None, None)),
            ('://?', (None, None, '://', '', None)),
            ('://#', (None, None, '://', None, '')),
            ('://?#', (None, None, '://', '', '')),
            ('//', (None, '', '', None, None)),
            ('///', (None, '', '/', None, None)),
            ('//?', (None, '', '', '', None)),
            ('//#', (None, '', '', None, '')),
            ('//?#', (None, '', '', '', '')),
            ('?', (None, None, '', '', None)),
            ('??', (None, None, '', '?', None)),
            ('?#', (None, None, '', '', '')),
            ('#', (None, None, '', None, '')),
            ('##', (None, None, '', None, '#')),
            (b'', (None, None, b'', None, None)),
            (b':', (None, None, b':', None, None)),
            (b':/', (None, None, b':/', None, None)),
            (b'://', (None, None, b'://', None, None)),
            (b'://?', (None, None, b'://', b'', None)),
            (b'://#', (None, None, b'://', None, b'')),
            (b'://?#', (None, None, b'://', b'', b'')),
            (b'//', (None, b'', b'', None, None)),
            (b'///', (None, b'', b'/', None, None)),
            (b'//?', (None, b'', b'', b'', None)),
            (b'//#', (None, b'', b'', None, b'')),
            (b'//?#', (None, b'', b'', b'', b'')),
            (b'?', (None, None, b'', b'', None)),
            (b'??', (None, None, b'', b'?', None)),
            (b'?#', (None, None, b'', b'', b'')),
            (b'#', (None, None, b'', None, b'')),
            (b'##', (None, None, b'', None, b'#')),
        ]
        for uri, parts in cases:
            self.check(uri, parts)

    def test_members(self):
        uri = 'foo://user@example.com:8042/over/there?name=ferret#nose'
        result = urisplit(uri)
        self.assertEqual(result.scheme, 'foo')
        self.assertEqual(result.authority, 'user@example.com:8042')
        self.assertEqual(result.path, '/over/there')
        self.assertEqual(result.query, 'name=ferret')
        self.assertEqual(result.fragment, 'nose')
        self.assertEqual(result.userinfo, 'user')
        self.assertEqual(result.host, 'example.com')
        self.assertEqual(result.port, '8042')
        self.assertEqual(result.geturi(), uri)
        self.assertEqual(result.getscheme(), 'foo')
        self.assertEqual(result.getuserinfo(), 'user')
        self.assertEqual(result.gethost(), 'example.com')
        self.assertEqual(result.getport(), 8042)
        self.assertEqual(result.getpath(), '/over/there')
        self.assertEqual(result.getquery(), 'name=ferret')
        self.assertEqual(dict(result.getquerydict()), {'name': ['ferret']})
        self.assertEqual(list(result.getquerylist()), [('name', 'ferret')])
        self.assertEqual(result.getfragment(), 'nose')

        uri = 'urn:example:animal:ferret:nose'
        result = urisplit(uri)
        self.assertEqual(result.scheme, 'urn')
        self.assertEqual(result.authority, None)
        self.assertEqual(result.path, 'example:animal:ferret:nose')
        self.assertEqual(result.query, None)
        self.assertEqual(result.fragment, None)
        self.assertEqual(result.userinfo, None)
        self.assertEqual(result.host, None)
        self.assertEqual(result.port, None)
        self.assertEqual(result.geturi(), uri)
        self.assertEqual(result.getscheme(), 'urn')
        self.assertEqual(result.getuserinfo(), None)
        self.assertEqual(result.gethost(), None)
        self.assertEqual(result.getport(), None)
        self.assertEqual(result.getpath(), 'example:animal:ferret:nose')
        self.assertEqual(result.getquery(), None)
        self.assertEqual(dict(result.getquerydict()), {})
        self.assertEqual(list(result.getquerylist()), [])
        self.assertEqual(result.getfragment(), None)

        uri = 'file:///'
        result = urisplit(uri)
        self.assertEqual(result.scheme, 'file')
        self.assertEqual(result.authority, '')
        self.assertEqual(result.path, '/')
        self.assertEqual(result.query, None)
        self.assertEqual(result.fragment, None)
        self.assertEqual(result.userinfo, None)
        self.assertEqual(result.host, '')
        self.assertEqual(result.port, None)
        self.assertEqual(result.geturi(), uri)
        self.assertEqual(result.getscheme(), 'file')
        self.assertEqual(result.getuserinfo(), None)
        self.assertEqual(result.gethost(), '')
        self.assertEqual(result.getport(), None)
        self.assertEqual(result.getpath(), '/')
        self.assertEqual(result.getquery(), None)
        self.assertEqual(dict(result.getquerydict()), {})
        self.assertEqual(list(result.getquerylist()), [])
        self.assertEqual(result.getfragment(), None)

        uri = b'foo://user@example.com:8042/over/there?name=ferret#nose'
        result = urisplit(uri)
        self.assertEqual(result.scheme, b'foo')
        self.assertEqual(result.authority, b'user@example.com:8042')
        self.assertEqual(result.path, b'/over/there')
        self.assertEqual(result.query, b'name=ferret')
        self.assertEqual(result.fragment, b'nose')
        self.assertEqual(result.userinfo, b'user')
        self.assertEqual(result.host, b'example.com')
        self.assertEqual(result.port, b'8042')
        self.assertEqual(result.geturi(), uri)
        self.assertEqual(result.getscheme(), 'foo')
        self.assertEqual(result.getuserinfo(), 'user')
        self.assertEqual(result.gethost(), 'example.com')
        self.assertEqual(result.getport(), 8042)
        self.assertEqual(result.getpath(), '/over/there')
        self.assertEqual(result.getquery(), 'name=ferret')
        self.assertEqual(dict(result.getquerydict()), {'name': ['ferret']})
        self.assertEqual(list(result.getquerylist()), [('name', 'ferret')])
        self.assertEqual(result.getfragment(), 'nose')

        uri = b'urn:example:animal:ferret:nose'
        result = urisplit(uri)
        self.assertEqual(result.scheme, b'urn')
        self.assertEqual(result.authority, None)
        self.assertEqual(result.path, b'example:animal:ferret:nose')
        self.assertEqual(result.query, None)
        self.assertEqual(result.fragment, None)
        self.assertEqual(result.userinfo, None)
        self.assertEqual(result.host, None)
        self.assertEqual(result.port, None)
        self.assertEqual(result.geturi(), uri)
        self.assertEqual(result.getscheme(), 'urn')
        self.assertEqual(result.getuserinfo(), None)
        self.assertEqual(result.gethost(), None)
        self.assertEqual(result.getport(), None)
        self.assertEqual(result.getpath(), 'example:animal:ferret:nose')
        self.assertEqual(result.getquery(), None)
        self.assertEqual(dict(result.getquerydict()), {})
        self.assertEqual(list(result.getquerylist()), [])
        self.assertEqual(result.getfragment(), None)

        uri = b'file:///'
        result = urisplit(uri)
        self.assertEqual(result.scheme, b'file')
        self.assertEqual(result.authority, b'')
        self.assertEqual(result.path, b'/')
        self.assertEqual(result.query, None)
        self.assertEqual(result.fragment, None)
        self.assertEqual(result.userinfo, None)
        self.assertEqual(result.host, b'')
        self.assertEqual(result.port, None)
        self.assertEqual(result.geturi(), uri)
        self.assertEqual(result.getscheme(), 'file')
        self.assertEqual(result.getuserinfo(), None)
        self.assertEqual(result.gethost(), '')
        self.assertEqual(result.getport(), None)
        self.assertEqual(result.getpath(), '/')
        self.assertEqual(result.getquery(), None)
        self.assertEqual(dict(result.getquerydict()), {})
        self.assertEqual(list(result.getquerylist()), [])
        self.assertEqual(result.getfragment(), None)

    def test_getscheme(self):
        self.assertEqual(urisplit('foo').getscheme(default='bar'), 'bar')
        self.assertEqual(urisplit('FOO_BAR:/').getscheme(), 'foo_bar')
        self.assertEqual(urisplit(b'foo').getscheme(default='bar'), 'bar')
        self.assertEqual(urisplit(b'FOO_BAR:/').getscheme(), 'foo_bar')

    def test_gethost(self):
        from ipaddress import IPv4Address, IPv6Address
        cases = [
            ('http://Test.python.org:5432/foo/', 'test.python.org'),
            ('http://12.34.56.78:5432/foo/', IPv4Address('12.34.56.78')),
            ('http://[::1]:5432/foo/', IPv6Address('::1')),
        ]
        for uri, host in cases:
            self.assertEqual(urisplit(uri).gethost(), host)
            self.assertEqual(urisplit(uri.encode()).gethost(), host)
        for uri in ['http://[::1/', 'http://::1]/']:
            with self.assertRaises(ValueError, msg='%r' % uri):
                urisplit(uri).gethost()
            with self.assertRaises(ValueError, msg='%r' % uri):
                urisplit(uri.encode()).gethost()

    def test_getport(self):
        for uri in ['foo://bar', 'foo://bar:', 'foo://bar/', 'foo://bar:/']:
            result = urisplit(uri)
            if result.authority.endswith(':'):
                self.assertEqual(result.port, '')
            else:
                self.assertEqual(result.port, None)
            self.assertEqual(result.gethost(), 'bar')
            self.assertEqual(result.getport(8000), 8000)

    def test_getpath(self):
        cases = [
            ('', '', '/'),
            ('.', './', '/'),
            ('./', './', '/'),
            ('./.', './', '/'),
            ('./..', '../', '/'),
            ('./foo', 'foo', '/foo'),
            ('./foo/', 'foo/', '/foo/'),
            ('./foo/.', 'foo/', '/foo/'),
            ('./foo/..', './', '/'),
            ('..', '../', '/'),
            ('../', '../', '/'),
            ('../.', '../', '/'),
            ('../..', '../../', '/'),
            ('../foo', '../foo', '/foo'),
            ('../foo/', '../foo/', '/foo/'),
            ('../foo/.', '../foo/', '/foo/'),
            ('../foo/..', '../', '/'),
            ('../../foo', '../../foo', '/foo'),
            ('../../foo/', '../../foo/', '/foo/'),
            ('../../foo/.', '../../foo/', '/foo/'),
            ('../../foo/..', '../../', '/'),
            ('../../foo/../bar', '../../bar', '/bar'),
            ('../../foo/../bar/', '../../bar/', '/bar/'),
            ('../../foo/../bar/.', '../../bar/', '/bar/'),
            ('../../foo/../bar/..', '../../', '/'),
            ('../../foo/../..', '../../../', '/')
        ]
        for uri, relpath, abspath in cases:
            parts = urisplit(uri)
            self.assertEqual(relpath, parts.getpath())
            parts = urisplit(uri.encode('ascii'))
            self.assertEqual(relpath, parts.getpath())
            parts = urisplit('/' + uri)
            self.assertEqual(abspath, parts.getpath())
            parts = urisplit(('/' + uri).encode('ascii'))
            self.assertEqual(abspath, parts.getpath())

    def test_getquery(self):
        cases = [
            ("?", [], {}),
            ("?&", [], {}),
            ("?&&", [], {}),
            ("?=",
             [('', '')],
             {'': ['']}),
            ("?=a",
             [('', 'a')],
             {'': ['a']}),
            ("?a",
             [('a', None)],
             {'a': [None]}),
            ("?a=",
             [('a', '')],
             {'a': ['']}),
            ("?&a=b",
             [('a', 'b')],
             {'a': ['b']}),
            ("?a=a+b&b=b+c",
             [('a', 'a+b'), ('b', 'b+c')],
             {'a': ['a+b'], 'b': ['b+c']}),
            ("?a=a%20b&b=b%20c",
             [('a', 'a b'), ('b', 'b c')],
             {'a': ['a b'], 'b': ['b c']}),
            ("?a=1&a=2",
             [('a', '1'), ('a', '2')],
             {'a': ['1', '2']}),
        ]
        for query, querylist, querydict in cases:
            self.assertEqual(urisplit(query).getquerylist(), querylist,
                             'Error parsing query dict for %r' % query)
            self.assertEqual(urisplit(query).getquerydict(), querydict,
                             'Error parsing query list for %r' % query)

    def test_ip_literal(self):
        cases = [
            ('http://Test.python.org:5432/foo/', 'test.python.org', 5432),
            ('http://12.34.56.78:5432/foo/', '12.34.56.78', 5432),
            ('http://[::1]:5432/foo/', '::1', 5432),
            ('http://[dead:beef::1]:5432/foo/', 'dead:beef::1', 5432),
            ('http://[dead:beef::]:5432/foo/', 'dead:beef::', 5432),
            ('http://[dead:beef:cafe:5417:affe:8FA3:deaf:feed]:5432/foo/',
             'dead:beef:cafe:5417:affe:8fa3:deaf:feed', 5432),
            ('http://[::12.34.56.78]:5432/foo/', '::c22:384e', 5432),
            ('http://[::ffff:12.34.56.78]:5432/foo/', '::ffff:c22:384e', 5432),
            ('http://Test.python.org/foo/', 'test.python.org', None),
            ('http://12.34.56.78/foo/', '12.34.56.78', None),
            ('http://[::1]/foo/', '::1', None),
            ('http://[dead:beef::1]/foo/', 'dead:beef::1', None),
            ('http://[dead:beef::]/foo/', 'dead:beef::', None),
            ('http://[dead:beef:cafe:5417:affe:8FA3:deaf:feed]/foo/',
             'dead:beef:cafe:5417:affe:8fa3:deaf:feed', None),
            ('http://[::12.34.56.78]/foo/', '::c22:384e', None),
            ('http://[::ffff:12.34.56.78]/foo/', '::ffff:c22:384e', None),
            ('http://Test.python.org:/foo/', 'test.python.org', None),
            ('http://12.34.56.78:/foo/', '12.34.56.78', None),
            ('http://[::1]:/foo/', '::1', None),
            ('http://[dead:beef::1]:/foo/', 'dead:beef::1', None),
            ('http://[dead:beef::]:/foo/', 'dead:beef::', None),
            ('http://[dead:beef:cafe:5417:affe:8FA3:deaf:feed]:/foo/',
             'dead:beef:cafe:5417:affe:8fa3:deaf:feed', None),
            ('http://[::12.34.56.78]:/foo/', '::c22:384e', None),
            ('http://[::ffff:12.34.56.78]:/foo/', '::ffff:c22:384e', None),
            ]
        for uri, host, port in cases:
            parts = urisplit(uri)
            self.assertEqual(host, str(parts.gethost()))
            self.assertEqual(port, parts.getport())
            parts = urisplit(uri.encode('ascii'))
            self.assertEqual(host, str(parts.gethost()))
            self.assertEqual(port, parts.getport())

    def test_invalid_ip_literal(self):
        uris = [
            'http://::12.34.56.78]/',
            'http://[::1/foo/',
            'ftp://[::1/foo/bad]/bad',
            'http://[::1/foo/bad]/bad',
            'http://[foo]/',
            'http://[v7.future]'
        ]
        for uri in uris:
            with self.assertRaises(ValueError, msg='%r' % uri):
                urisplit(uri).gethost()
            with self.assertRaises(ValueError, msg='%r' % uri.encode('ascii')):
                urisplit(uri.encode('ascii')).gethost()



import sys

from setuptools import find_packages, setup

# environment markers require a recent setuptools and/or pip version
if sys.version_info >= (3, 3) or 'bdist_wheel' in sys.argv:
    install_requires = []
elif sys.version_info >= (3, 0):
    install_requires = ['ipaddress>=1.0.7']
else:
    install_requires = ['ipaddress>=1.0.6']


def get_version(filename):
    from re import findall
    with open(filename) as f:
        metadata = dict(findall("__([a-z]+)__ = '([^']+)'", f.read()))
    return metadata['version']

setup(
    name='uritools',
    version=get_version('uritools/__init__.py'),
    url='https://github.com/tkem/uritools/',
    license='MIT',
    author='Thomas Kemmer',
    author_email='tkemmer@computer.org',
    description=(
        'RFC 3986 compliant, Unicode-aware, scheme-agnostic '
        'replacement for urlparse'
    ),
    long_description=open('README.rst').read(),
    keywords='uri url urlparse urlsplit urljoin urldefrag',
    packages=find_packages(exclude=['tests', 'tests.*']),
    install_requires=install_requires,
    extras_require={
        ':python_version == "2.7"': ['ipaddress>=1.0.6'],
        ':python_version == "3.2"': ['ipaddress>=1.0.7']
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Other Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.2',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Internet',
        'Topic :: Software Development :: Libraries :: Python Modules'
    ]
)
